@@ -3,31 +3,33 @@
     unit test for the filters
     ~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Missing tests:
-
-    -   wordcount
-    -   rst
-    -   markdown
-    -   textile
-
-    :copyright: 2007 by Armin Ronacher.
+    :copyright: 2008 by Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
+from jinja2 import Markup, Environment
+
 
 CAPITALIZE = '''{{ "foo bar"|capitalize }}'''
-CAPTURE = '''{{ "foo"|capture('bar') }}|{{ bar }}'''
 CENTER = '''{{ "foo"|center(9) }}'''
 DEFAULT = '''{{ missing|default("no") }}|{{ false|default('no') }}|\
 {{ false|default('no', true) }}|{{ given|default("no") }}'''
 DICTSORT = '''{{ foo|dictsort }}|\
 {{ foo|dictsort(true) }}|\
 {{ foo|dictsort(false, 'value') }}'''
-ESCAPE = '''{{ '<">&'|escape }}|{{ '<">&'|escape(true) }}'''
+BATCH = '''{{ foo|batch(3)|list }}|{{ foo|batch(3, 'X')|list }}'''
+SLICE = '''{{ foo|slice(3)|list }}|{{ foo|slice(3, 'X')|list }}'''
+ESCAPE = '''{{ '<">&'|escape }}'''
+STRIPTAGS = '''{{ foo|striptags }}'''
 FILESIZEFORMAT = '{{ 100|filesizeformat }}|\
 {{ 1000|filesizeformat }}|\
 {{ 1000000|filesizeformat }}|\
 {{ 1000000000|filesizeformat }}|\
-{{ 1000000000000|filesizeformat }}'
+{{ 1000000000000|filesizeformat }}|\
+{{ 100|filesizeformat(true) }}|\
+{{ 1000|filesizeformat(true) }}|\
+{{ 1000000|filesizeformat(true) }}|\
+{{ 1000000000|filesizeformat(true) }}|\
+{{ 1000000000000|filesizeformat(true) }}'
 FIRST = '''{{ foo|first }}'''
 FLOAT = '''{{ "42"|float }}|{{ "ajsghasjgd"|float }}|{{ "32.32"|float }}'''
 FORMAT = '''{{ "%s|%s"|format("a", "b") }}'''
@@ -39,7 +41,7 @@ LENGTH = '''{{ "hello world"|length }}'''
 LOWER = '''{{ "FOO"|lower }}'''
 PPRINT = '''{{ data|pprint }}'''
 RANDOM = '''{{ seq|random }}'''
-REVERSE = '''{{ "foobar"|reverse }}|{{ [1, 2, 3]|reverse }}'''
+REVERSE = '''{{ "foobar"|reverse|join }}|{{ [1, 2, 3]|reverse|list }}'''
 STRING = '''{{ range(10)|string }}'''
 TITLE = '''{{ "foo bar"|title }}'''
 TRIM = '''{{ "      foo       "|trim }}'''
@@ -47,11 +49,25 @@ TRUNCATE = '''{{ data|truncate(15, true, ">>>") }}|\
 {{ data|truncate(15, false, ">>>") }}|\
 {{ smalldata|truncate(15) }}'''
 UPPER = '''{{ "foo"|upper }}'''
-URLENCODE = '''{{ "f#b"|urlencode }}'''
 URLIZE = '''{{ "foo http://www.example.com/ bar"|urlize }}'''
 WORDCOUNT = '''{{ "foo bar baz"|wordcount }}'''
 BLOCK = '''{% filter lower|escape %}<HEHE>{% endfilter %}'''
 CHAINING = '''{{ ['<foo>', '<bar>']|first|upper|escape }}'''
+SUM = '''{{ [1, 2, 3, 4, 5, 6]|sum }}'''
+ABS = '''{{ -1|abs }}|{{ 1|abs }}'''
+ROUND = '''{{ 2.7|round }}|{{ 2.1|round }}|\
+{{ 2.1234|round(2, 'floor') }}|{{ 2.1|round(0, 'ceil') }}'''
+XMLATTR = '''{{ {'foo': 42, 'bar': 23, 'fish': none,
+'spam': missing, 'blub:blub': '<?>'}|xmlattr }}'''
+SORT = '''{{ [2, 3, 1]|sort }}|{{ [2, 3, 1]|sort(true) }}'''
+GROUPBY = '''{% for grouper, list in [{'foo': 1, 'bar': 2},
+                 {'foo': 2, 'bar': 3},
+                 {'foo': 1, 'bar': 1},
+                 {'foo': 3, 'bar': 4}]|groupby('foo') -%}
+{{ grouper }}: {{ list|join(', ') }}
+{% endfor %}'''
+FILTERTAG = '''{% filter upper|replace('FOO', 'foo') %}foobar{% endfilter %}'''
+SORT = '''{{ ['foo', 'Bar', 'blah']|sort }}'''
 
 
 def test_capitalize(env):
@@ -59,11 +75,6 @@ def test_capitalize(env):
     assert tmpl.render() == 'Foo bar'
 
 
-def test_capture(env):
-    tmpl = env.from_string(CAPTURE)
-    assert tmpl.render() == 'foo|foo'
-
-
 def test_center(env):
     tmpl = env.from_string(CENTER)
     assert tmpl.render() == '   foo   '
@@ -82,16 +93,41 @@ def test_dictsort(env):
                    "[('a', 0), ('b', 1), ('c', 2), ('A', 3)]")
 
 
+def test_batch(env):
+    tmpl = env.from_string(BATCH)
+    out = tmpl.render(foo=range(10))
+    assert out == ("[[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]|"
+                   "[[0, 1, 2], [3, 4, 5], [6, 7, 8], [9, 'X', 'X']]")
+
+
+def test_slice(env):
+    tmpl = env.from_string(SLICE)
+    out = tmpl.render(foo=range(10))
+    assert out == ("[[0, 1, 2, 3], [4, 5, 6], [7, 8, 9]]|"
+                   "[[0, 1, 2, 3], [4, 5, 6, 'X'], [7, 8, 9, 'X']]")
+
+
 def test_escape(env):
     tmpl = env.from_string(ESCAPE)
     out = tmpl.render()
-    assert out == '&lt;"&gt;&amp;|&lt;&quot;&gt;&amp;'
+    assert out == '&lt;&#34;&gt;&amp;'
+
+
+def test_striptags(env):
+    tmpl = env.from_string(STRIPTAGS)
+    out = tmpl.render(foo='  <p>just a small   \n <a href="#">'
+                      'example</a> link</p>\n<p>to a webpage</p> '
+                      '<!-- <p>and some commented stuff</p> -->')
+    assert out == 'just a small example link to a webpage'
 
 
 def test_filesizeformat(env):
     tmpl = env.from_string(FILESIZEFORMAT)
     out = tmpl.render()
-    assert out == '100 Bytes|1000 Bytes|976.6 KB|953.7 MB|931.3 GB'
+    assert out == (
+        '100 Bytes|1.0 KB|1.0 MB|1.0 GB|1000.0 GB|'
+        '100 Bytes|1000 Bytes|976.6 KiB|953.7 MiB|931.3 GiB'
+    )
 
 
 def test_first(env):
@@ -116,8 +152,8 @@ def test_indent(env):
     tmpl = env.from_string(INDENT)
     text = '\n'.join([' '.join(['foo', 'bar'] * 2)] * 2)
     out = tmpl.render(foo=text)
-    assert out == 'foo bar foo bar\n  foo bar foo bar|  ' \
-                  'foo bar foo bar\n  foo bar foo bar'
+    assert out == ('foo bar foo bar\n  foo bar foo bar|  '
+                   'foo bar foo bar\n  foo bar foo bar')
 
 
 def test_int(env):
@@ -131,6 +167,10 @@ def test_join(env):
     out = tmpl.render()
     assert out == '1|2|3'
 
+    env2 = Environment(autoescape=True)
+    tmpl = env2.from_string('{{ ["<foo>", "<span>foo</span>"|safe]|join }}')
+    assert tmpl.render() == '&lt;foo&gt;<span>foo</span>'
+
 
 def test_last(env):
     tmpl = env.from_string(LAST)
@@ -153,7 +193,7 @@ def test_lower(env):
 def test_pprint(env):
     from pprint import pformat
     tmpl = env.from_string(PPRINT)
-    data = range(10000)
+    data = range(1000)
     assert tmpl.render(data=data) == pformat(data)
 
 
@@ -171,7 +211,7 @@ def test_reverse(env):
 
 def test_string(env):
     tmpl = env.from_string(STRING)
-    assert tmpl.render(foo=range(10)) == str(range(10))
+    assert tmpl.render(foo=range(10)) == unicode(xrange(10))
 
 
 def test_title(env):
@@ -196,11 +236,6 @@ def test_upper(env):
     assert tmpl.render() == 'FOO'
 
 
-def test_urlencode(env):
-    tmpl = env.from_string(URLENCODE)
-    assert tmpl.render() == 'f%23b'
-
-
 def test_urlize(env):
     tmpl = env.from_string(URLIZE)
     assert tmpl.render() == 'foo <a href="http://www.example.com/">'\
@@ -220,3 +255,77 @@ def test_block(env):
 def test_chaining(env):
     tmpl = env.from_string(CHAINING)
     assert tmpl.render() == '&lt;FOO&gt;'
+
+
+def test_sum(env):
+    tmpl = env.from_string(SUM)
+    assert tmpl.render() == '21'
+
+
+def test_abs(env):
+    tmpl = env.from_string(ABS)
+    return tmpl.render() == '1|1'
+
+
+def test_round(env):
+    tmpl = env.from_string(ROUND)
+    return tmpl.render() == '3.0|2.0|2.1|3.0'
+
+
+def test_xmlattr(env):
+    tmpl = env.from_string(XMLATTR)
+    out = tmpl.render().split()
+    assert len(out) == 3
+    assert 'foo="42"' in out
+    assert 'bar="23"' in out
+    assert 'blub:blub="&lt;?&gt;"' in out
+
+
+def test_sort(env):
+    tmpl = env.from_string(SORT)
+    assert tmpl.render() == '[1, 2, 3]|[3, 2, 1]'
+
+
+def test_groupby(env):
+    tmpl = env.from_string(GROUPBY)
+    assert tmpl.render().splitlines() == [
+        "1: {'foo': 1, 'bar': 2}, {'foo': 1, 'bar': 1}",
+        "2: {'foo': 2, 'bar': 3}",
+        "3: {'foo': 3, 'bar': 4}"
+    ]
+
+
+def test_filtertag(env):
+    tmpl = env.from_string(FILTERTAG)
+    assert tmpl.render() == 'fooBAR'
+
+
+def test_replace():
+    env = Environment()
+    tmpl = env.from_string('{{ string|replace("o", 42) }}')
+    assert tmpl.render(string='<foo>') == '<f4242>'
+
+    env = Environment(autoescape=True)
+    tmpl = env.from_string('{{ string|replace("o", 42) }}')
+    assert tmpl.render(string='<foo>') == '&lt;f4242&gt;'
+    tmpl = env.from_string('{{ string|replace("<", 42) }}')
+    assert tmpl.render(string='<foo>') == '42foo&gt;'
+    tmpl = env.from_string('{{ string|replace("o", ">x<") }}')
+    assert tmpl.render(string=Markup('foo')) == 'f&gt;x&lt;&gt;x&lt;'
+
+
+def test_forceescape(env):
+    tmpl = env.from_string('{{ x|forceescape }}')
+    assert tmpl.render(x=Markup('<div />')) == u'&lt;div /&gt;'
+
+
+def test_safe():
+    env = Environment(autoescape=True)
+    tmpl = env.from_string('{{ "<div>foo</div>"|safe }}')
+    assert tmpl.render() == '<div>foo</div>'
+    tmpl = env.from_string('{{ "<div>foo</div>" }}')
+    assert tmpl.render() == '&lt;div&gt;foo&lt;/div&gt;'
+
+
+def test_sort(env):
+    assert env.from_string(SORT).render() == "['Bar', 'blah', 'foo']"
